const { Builder, By, until } = require("selenium-webdriver");
const assert = require("assert");

//const userApi = require("../backend/userapi");
//const { retrieveJWT } = require("../JWTManager");

describe("User Managment Test", function () {
  let driver;

  beforeEach(async function () {
    driver = await new Builder().forBrowser("chrome").build();
    //navigate to the application
    await driver.get("http://localhost:3000");
    await driver.manage().window().maximize();

    await driver.findElement(By.id("loginButton")).click();
    await driver.wait(until.elementLocated(By.id("loginDialog")), 5000);

    const isDialogOpen = await driver.findElement(By.id("loginDialog")).isDisplayed();
    assert.strictEqual(isDialogOpen, true, "Der Dialog wurde nicht geöffnet.");

    await driver.findElement(By.id("inputEmail")).sendKeys("john@some-host.de");
    await driver.findElement(By.id("inputPassword")).sendKeys("12abcAB!");

    await driver.findElement(By.id("okButton")).click();

    await driver.wait(until.elementLocated(By.id("logoutButton")), 5000);
    const isLogoutButtonOpen = await driver.findElement(By.id("logoutButton")).isDisplayed();
    assert.strictEqual(isLogoutButtonOpen, true, "Der Logout Button wird nicht angezeigt.");
    
    //click UserManagment
    driver.findElement(By.linkText("UserManagment")).click();
    await driver.wait(until.elementLocated(By.id("listUsers")), 5000);
    const listUsers = await driver.findElement(By.id("listUsers")).isDisplayed();
    assert.strictEqual(listUsers, true, "Der Dialog wurde nicht geöffnet.");
  },10000);

  afterEach(async function () {
    await driver.quit();
  });

  it("listing all users and admins", async function (){
    const userJohn = await driver.findElement(By.id("user-John")).isDisplayed();
    assert.strictEqual(userJohn, true, "Der User John wurde nicht gefunden.");
    const userFlo = await driver.findElement(By.id("user-Flo")).isDisplayed();
    assert.strictEqual(userFlo, true, "Der User Flo wurde nicht gefunden.");
  },20000);

  it("create user", async function (){
    driver.findElement(By.id("addUserButton")).click();
    await driver.wait(until.elementLocated(By.id("addUserDialog")), 5000);

    const isDialogOpen = await driver.findElement(By.id("addUserDialog")).isDisplayed();
    assert.strictEqual(isDialogOpen, true, "Der Dialog wurde nicht geöffnet.");

    await driver.findElement(By.id("inputUsername")).sendKeys("Max");
    await driver.findElement(By.id("inputEmail")).sendKeys("max@some-host.de");
    await driver.findElement(By.id("inputPassword")).sendKeys("12abcAB!");

    await driver.findElement(By.id("addButton")).click();

    await driver.wait(until.elementLocated(By.id("user-Max")), 5000);
    const userMax = await driver.findElement(By.id("user-Max")).isDisplayed();
    assert.strictEqual(userMax, true, "Der User Max wurde nicht gefunden.");
  },10000);

  /*
  it("create admin", async function (){
    driver.findElement(By.id("addUserButton")).click();
    await driver.wait(until.elementLocated(By.id("addUserDialog")), 5000);

    const isDialogOpen = await driver.findElement(By.id("addUserDialog")).isDisplayed();
    assert.strictEqual(isDialogOpen, true, "Der Dialog wurde nicht geöffnet.");

    await driver.findElement(By.id("inputUsername")).sendKeys("MaxAdmin");
    await driver.findElement(By.id("inputEmail")).sendKeys("maxAdmin@some-host.de");
    await driver.findElement(By.id("inputPassword")).sendKeys("12abcAB!");
    await driver.findElement(By.id("formAdminCheckbox")).click();

    await driver.findElement(By.id("addButton")).click();

    await driver.wait(until.elementLocated(By.id("user-MaxAdmin")), 5000);
    const userMaxAdmin = await driver.findElement(By.id("user-MaxAdmin")).isDisplayed();
    assert.strictEqual(userMaxAdmin, true, "Der Admin Max wurde nicht gefunden.");
  },10000);
  */

  it("edit user", async function (){
    driver.findElement(By.id("editButton-Flo")).click();
    await driver.wait(until.elementLocated(By.id("editUserDialog")), 5000);
    const isDialogOpen = await driver.findElement(By.id("editUserDialog")).isDisplayed();
    assert.strictEqual(isDialogOpen, true, "Der Dialog wurde nicht geöffnet.");

    const usernameInput = await driver.findElement(By.id("inputUsername"));
    await usernameInput.clear(); 
    await usernameInput.sendKeys("Florian");
    const emailInput = await driver.findElement(By.id("inputEmail"));
    await emailInput.clear(); 
    await emailInput.sendKeys("florian@some-host.de")

    await driver.findElement(By.id("editNow")).click();

    await driver.wait(until.elementLocated(By.id("user-Florian")), 5000);
    const userFlorianAdmin = await driver.findElement(By.id("user-Florian")).isDisplayed();
    assert.strictEqual(userFlorianAdmin, true, "Der User Florian wurde nicht gefunden.");
  },10000);

  it("delete user", async function (){
    await driver.findElement(By.id("deleteButton-Max")).click();
    await driver.sleep(500);

    // Überprüfen das "Max" nicht mehr in der Benutzerliste vorhanden ist
    const isMaxDeleted = await driver.findElement(By.id("user-Max")).isDisplayed().then(
      () => false,
      (error) => error.toString().includes("no such element"),
    );
    assert.strictEqual(isMaxDeleted, true, "Der Benutzer 'Max' wurde nicht gelöscht.");
  },10000);

  it("login with new account", async function (){
    driver.findElement(By.id("addUserButton")).click();
    await driver.wait(until.elementLocated(By.id("addUserDialog")), 5000);

    const isAddDialogOpen = await driver.findElement(By.id("addUserDialog")).isDisplayed();
    assert.strictEqual(isAddDialogOpen, true, "Der Dialog wurde nicht geöffnet.");
    //Informationen in Dialog eintragen
    await driver.findElement(By.id("inputUsername")).sendKeys("Max");
    await driver.findElement(By.id("inputEmail")).sendKeys("max@some-host.de");
    await driver.findElement(By.id("inputPassword")).sendKeys("12abcAB!");

    await driver.findElement(By.id("addButton")).click();

    //Check ob User erstellt
    await driver.wait(until.elementLocated(By.id("user-Max")), 5000);
    const userMax = await driver.findElement(By.id("user-Max")).isDisplayed();
    assert.strictEqual(userMax, true, "Der User Max wurde nicht gefunden.");

    //Ausloggen
    await driver.findElement(By.id("logoutButton")).click();
    console.log("Logout erfolgreich");
    await driver.sleep(500);

    //Login mit neuen User
    await driver.findElement(By.id("loginButton")).click();
    await driver.wait(until.elementLocated(By.id("loginDialog")), 5000);
    const isLoginDialogOpen = await driver.findElement(By.id("loginDialog")).isDisplayed();
    assert.strictEqual(isLoginDialogOpen, true, "Der Dialog wurde nicht geöffnet.");

    const emailInput = await driver.findElement(By.id("inputEmail"));
    const passwordInput = await driver.findElement(By.id("inputPassword"));
    await emailInput.sendKeys("max@some-host.de");
    await passwordInput.sendKeys("12abcAB!");
    await driver.findElement(By.id("okButton")).click();


    //Check ob eingeloggt
    await driver.wait(until.elementLocated(By.id("logoutButton")), 5000);
    const isLogoutButtonOpenMax = await driver.findElement(By.id("logoutButton")).isDisplayed();
    assert.strictEqual(isLogoutButtonOpenMax, true, "Der Logout Button Max wird nicht angezeigt.");
  },10000);

  it("create User without password ERROR!", async function (){
    driver.findElement(By.id("addUserButton")).click();
    await driver.wait(until.elementLocated(By.id("addUserDialog")), 5000);

    const isDialogOpen = await driver.findElement(By.id("addUserDialog")).isDisplayed();
    assert.strictEqual(isDialogOpen, true, "Der Dialog wurde nicht geöffnet.");

    await driver.findElement(By.id("inputUsername")).sendKeys("Tim");
    await driver.findElement(By.id("inputEmail")).sendKeys("tim@some-host.de");
    await driver.findElement(By.id("inputPassword")).sendKeys("");

    await driver.findElement(By.id("addButton")).click();

    await driver.wait(until.elementLocated(By.id("warningMessage")), 5000);
    const warningMessage = await driver.findElement(By.id("warningMessage")).isDisplayed();
    assert.strictEqual(warningMessage, true, "Die Fehlermeldung wurde nicht gefunden.");
  },10000);

  it("clean up test", async function (){
    console.log("Start afterAll");
    //Löschen von User Max
    await driver.findElement(By.id("deleteButton-Max")).click();

    await driver.findElement(By.id("editButton-Florian")).click();
    await driver.wait(until.elementLocated(By.id("editUserDialog")), 5000);
    const isDialogOpen = await driver.findElement(By.id("editUserDialog")).isDisplayed();
    assert.strictEqual(isDialogOpen, true, "Der Dialog wurde nicht geöffnet.");

    const usernameInput = await driver.findElement(By.id("inputUsername"));
    await usernameInput.clear(); 
    await usernameInput.sendKeys("Flo");
    const emailInput = await driver.findElement(By.id("inputEmail"));
    await emailInput.clear(); 
    await emailInput.sendKeys("flo@some-host.de")

    await driver.findElement(By.id("editNow")).click();

    await driver.wait(until.elementLocated(By.id("user-Flo")), 5000);
    const userFlorianAdmin = await driver.findElement(By.id("user-Flo")).isDisplayed();
    assert.strictEqual(userFlorianAdmin, true, "Der User Flo wurde nicht gefunden.");
  },10000);
});
